@@ -25,7 +25,6 @@
 #include "util.h"
 #include "version.h"
 
-
 unsigned char init_palette_values[NUM_PALETTE_ENTRIES][3] = 
 { {255, 255, 255}, 
   {190, 190, 190}, 
@@ -122,9 +121,14 @@ static void free_image_buf(image_buf* ibuf)
 }
 
 const char *initial_font_name = "-*-helvetica-*-r-normal--*-120-*-*-*-*-iso8859-1";
+const int white_pixel = 255;
+const int black_palette = 14;
+const int white_palette = 0;
+
 static void image_buf_create_pixmap(image_buf *ibuf)
 {
    int width, height;
+
    g_assert(ibuf);
    g_assert(ibuf->window);
    g_assert(ibuf->pixmap == 0);
@@ -139,17 +143,23 @@ static void image_buf_create_pixmap(image_buf *ibuf)
    {
       ibuf->rgbbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, width, height);
       g_assert(ibuf->rgbbuf);
-      /*memset(image_buf_rgbbuf(ibuf), 0, image_buf_rowstride(ibuf) * image_buf_height(ibuf) * sizeof(unsigned char));
-      */
+
+      /* set the initial image to white, so it looks like a blank sheet of paper */
+      memset(image_buf_rgbbuf(ibuf), white_pixel, 
+        image_buf_rowstride(ibuf) * image_buf_height(ibuf) * sizeof(unsigned char));
+      
       ibuf->pixmap = gdk_pixmap_new(ibuf->window->window, image_buf_width(ibuf), image_buf_height(ibuf), -1);
       g_assert(ibuf->pixmap);
       ibuf->gc = gdk_gc_new(ibuf->pixmap);
       g_assert(ibuf->gc);
+
       ibuf->cursor = 0;
       ibuf->filled = 0;
       ibuf->flash_state = 0;
       image_buf_set_font(ibuf, gdk_font_load(initial_font_name), initial_font_name);
       image_buf_rgbbuf_to_pixmap(ibuf, 0);
+      image_buf_set_foreground_to_palette(ibuf, black_palette);
+      image_buf_set_background_to_palette(ibuf, white_palette);
       memset(ibuf->textbuf, 0, sizeof(ibuf->textbuf));
       ibuf->timer = gtk_timeout_add(500, (GtkFunction)(handle_timeout), ibuf);
    }
@@ -216,11 +226,15 @@ GHashTable *images = 0;
 
 image_buf *create_image_buf(const char *name, int width, int height)
 {
+   const int window_width = MIN(width+120, 740);
+   const int window_height = MIN(height+220, 680);
+
    GtkWidget *widget = create_mainwindow();
    image_buf *ibuf;
    GtkWidget *scroll_frame = lookup_widget(widget, "scroll_frame");
    create_scroll_frame_widget_content(scroll_frame, widget);
    
+   gtk_window_set_default_size(GTK_WINDOW(widget), window_width, window_height); 
    gtk_widget_show (widget);
       
    ibuf = make_image_buf(width, height);
@@ -542,24 +556,66 @@ int image_buf_has_focus(image_buf *ibuf)
    return ibuf->has_focus;
 }
 
-
-void image_buf_set_foreground_to_palette(image_buf *ibuf, int ic)
+/*
+ * Convert the GdkColor object associated with a palette into
+ * a flat 24-bit rgb integer. See set_palette_entries().
+ */
+static guint32 image_buf_get_palette_color(image_buf *ibuf, int ic)
 {
-   GnomeColorPicker *gnomecolorpicker = (GnomeColorPicker*) lookup_widget(ibuf->window, "foreground_color_picker");			
-   short a;
-   guint32 color;
+   guint32 red, green, blue;
    g_assert(ibuf);
-   g_assert(gnomecolorpicker);
    g_assert(ic >= 0);
    g_assert(ic < NUM_PALETTE_ENTRIES);
-   ibuf->current_palette = ic;
+
+   red   = ibuf->palettes[ic].red   >> 8;    /* convert 16-bits into 8-bits */
+   green = ibuf->palettes[ic].green >> 8; 
+   blue  = ibuf->palettes[ic].blue  >> 8;
+   return (red << 16) | (green << 8) | blue; /* pack into one integer */
+}
+
+/*
+ * Change a color picker to the color of the given palette.
+ */
+static void image_buf_set_picker(image_buf *ibuf, int ic, const char *widget_name)
+{
+   GnomeColorPicker *color_picker;
+   g_assert(ibuf);
+   g_assert(ic >= 0);
+   g_assert(ic < NUM_PALETTE_ENTRIES);
+   g_assert(widget_name);
    
-   color = ibuf->palettes[ic].red >> 8 << 16 | ibuf->palettes[ic].green >> 8 << 8 | ibuf->palettes[ic].blue >> 8;
+   color_picker = (GnomeColorPicker*) lookup_widget(ibuf->window, widget_name);			
+   g_assert(color_picker);
+   gnome_color_picker_set_i16(color_picker, ibuf->palettes[ic].red, ibuf->palettes[ic].green, ibuf->palettes[ic].blue, 0);
+}  
+
+/*
+ * Change the current foreground color.
+ */
+void image_buf_set_foreground_to_palette(image_buf *ibuf, int ic)
+{
+   guint32 color;
+   g_assert(ibuf);
+
+   color = image_buf_get_palette_color(ibuf, ic);
    gdk_rgb_gc_set_foreground(ibuf->gc, color);
-   gnome_color_picker_set_i16(gnomecolorpicker, ibuf->palettes[ic].red, ibuf->palettes[ic].green, ibuf->palettes[ic].blue, a);
+   image_buf_set_picker(ibuf, ic, "foreground_color_picker");
+   ibuf->current_palette = ic;
 }   
 
+/*
+ * Change the current background color.
+ */
+void image_buf_set_background_to_palette(image_buf *ibuf, int ic)
+{
+   guint32 color;
+   g_assert(ibuf);
 
+   color = image_buf_get_palette_color(ibuf, ic);
+   gdk_rgb_gc_set_background(ibuf->gc, color);
+   image_buf_set_picker(ibuf, ic, "background_color_picker");
+}
+    
 void image_buf_set_pts_top_left(image_buf * ibuf, int x, int y)
 {
    GdkRectangle rect;
@@ -696,7 +752,7 @@ void image_buf_get_selected_region_to_pixbuf(image_buf *ibuf, GdkPixbuf **pixbuf
    unsigned char alphaonly[4] = {0, 0, 0, 1};
    unsigned char *data = 0, *c, *d;
    unsigned char default_pixel[4] = {0, 0, 0, 255};
-   int x, y, i;
+   int x, y;
    int pixel_size;
    rect = compute_cover_rect(ibuf->pts, ibuf->num_pts);
    
