@@ -33,6 +33,15 @@
 GdkCursor *busy_cursor = 0;
 GdkCursor *arrow_cursor = 0;
 
+/* FIXME: brush objects */
+double brush_spacing = 6.0;        
+int erase_size = 4;
+int brush_size = 9;
+const double EPSILON = 0.00001;
+double brush_distance = 0;
+int brush_last_x = 0;
+int brush_last_y = 0;
+
 
 void image_buf_force_select_region(image_buf *ibuf)
 {
@@ -58,9 +67,6 @@ void image_buf_force_select_region(image_buf *ibuf)
 }    
 
 
-const int erase_size = 4;
-
-
 
 GtkWidget *widget_get_toplevel_parent(GtkWidget *widget)
 {
@@ -832,6 +838,8 @@ void handle_button_press(image_buf *ibuf, int x, int y)
       break;
    	
    case ERASE:
+      /* TODO: object-oriented brush */
+      brush_spacing = 2.0;        
       gdk_gc_set_foreground(ibuf->gc, &(gcvalues.background));
       draw_rectangle(ibuf->pixmap, ibuf->gc, TRUE, x - erase_size, y - erase_size, x + erase_size, y + erase_size);
       draw_rectangle(ibuf->drawing_area->window, ibuf->gc, TRUE, x - erase_size, y - erase_size, x + erase_size, y + erase_size);
@@ -861,21 +869,27 @@ void handle_button_press(image_buf *ibuf, int x, int y)
       break;
    
    case BRUSH:
-      /* brain dead brush */
-      draw_rectangle(ibuf->pixmap, ibuf->gc, TRUE, 
-                     x - erase_size, y - erase_size, x + erase_size, y + erase_size);
-      draw_rectangle(ibuf->drawing_area->window, ibuf->gc, TRUE,
-                     x - erase_size, y - erase_size, x + erase_size, y + erase_size);
+      /* TODO: object-oriented brush */
+      brush_spacing = 6.0;        
+      draw_oval(ibuf->pixmap, ibuf->gc, 
+                TRUE, x - brush_size, y - brush_size, x + brush_size, y + brush_size);
+      draw_oval(ibuf->drawing_area->window, ibuf->gc, 
+                TRUE, x - brush_size, y - brush_size, x + brush_size, y + brush_size);
       IMAGE_MODIFIED;
       break;
                 
    default:
       gdk_beep();
    }
+
+   brush_distance = 0;
+   brush_last_x = x;
+   brush_last_y = y;
+
    ibuf->lx = x;
    ibuf->ly = y;
-   ibuf->mx = x; /* INT_MIN; */
-   ibuf->my = y; /* INT_MIN; */
+   ibuf->mx = INT_MIN;
+   ibuf->my = INT_MIN; 
 }   
 
 void handle_button_move(image_buf *ibuf, int x, int y)
@@ -1281,55 +1295,86 @@ int handle_timeout(image_buf *ibuf)
    return 1;
 }
 
+/* 
+ * Paint along the mouse movement. 
+ * adapted from paint_core.c in the GIMP.
+ */
 void image_buf_paint_interpolate(image_buf *ibuf, int x, int y)
 {
-   double dx;                /* delta x */
-   double dy;                /* delta y */
-   double lastx = ibuf->mx;  
-   double lasty = ibuf->my;
-   double distance;          /* distance moved */
-   double s = 0.0;           /* distance painted */
-   double step = 1.0;        /* paint spacing */
-   double factor;            /* ratio of painted to moved */
-   int    changed = 0;       /* true if image has been changed */
+   int    initial_x = x;
+   int    initial_y = y;
+
+   double dx;         /* delta x */
+   double dy;         /* delta y */
+   double moved;      /* mouse movement */       
+   double initial;    /* initial paint distance */
+   double final;      /* final paint distance   */
+
+   double points;     /* number of paint points */
+   double next;       /* distance to the next point */
+   double percent;    /* linear interplotation, 0 to 1.0 */   
+   int    changed = 0;
 
    /* TODO: object-oriented drawing tools */
    GdkGCValues gcvalues;
    gdk_gc_get_values(ibuf->gc, &gcvalues);
-
-   /* calculate distance moved: horizontal, vertical, and diag. */
-   dx = (double)(x - ibuf->mx);
-   dy = (double)(y - ibuf->my);
-   distance = sqrt(dx*dx + dy*dy);
-
-   /* return if no movement. */
-   if (distance < 1.0) return;
-
-   /* TODO: object-oriented drawing tools */
    if (ibuf->current_tool==ERASE)
    {
       gdk_gc_set_foreground(ibuf->gc, &(gcvalues.background));
    }
 
-   /* paint along the movement. */
-   while ( s < distance ) {
-      s += step;          
-      factor = s / distance;
-      x = (int)(lastx + factor * dx);
-      y = (int)(lasty + factor * dy);
+   /* calculate mouse move distance */
+   dx = (double)(x - brush_last_x);
+   dy = (double)(y - brush_last_y);
+   moved = sqrt(dx*dx + dy*dy);
 
-      /* TODO: object-oriented drawing tools */
-      draw_rectangle(ibuf->pixmap, ibuf->gc, 
-                     TRUE, x - erase_size, y - erase_size, x + erase_size, y + erase_size);
-      draw_rectangle(ibuf->drawing_area->window, ibuf->gc, 
-                     TRUE, x - erase_size, y - erase_size, x + erase_size, y + erase_size);
+   initial = brush_distance;
+   final = initial + moved;
 
+   /* paint along the movement */
+   while (brush_distance < final) {
+
+      /* calculate distance to the next point */
+      points = (int) (brush_distance / brush_spacing + 1.0 + EPSILON); 
+      next = points * brush_spacing - brush_distance;
+      brush_distance += next;          
+
+      if (brush_distance <= (final + EPSILON)) {   
+    
+         /* calculate interpolation */ 
+         percent = (brush_distance - initial) / moved;
+         x = (int)(brush_last_x + percent * dx);
+         y = (int)(brush_last_y + percent * dy);
+
+         /* TODO: object-oriented drawing tools */
+         switch (ibuf->current_tool) {
+            case ERASE:
+               draw_rectangle(ibuf->pixmap, ibuf->gc, 
+                        TRUE, x - erase_size, y - erase_size, x + erase_size, y + erase_size);
+               draw_rectangle(ibuf->drawing_area->window, ibuf->gc, 
+                        TRUE, x - erase_size, y - erase_size, x + erase_size, y + erase_size);
+               break;
+   
+            case BRUSH:   
+               draw_oval(ibuf->pixmap, ibuf->gc, 
+                       TRUE, x - brush_size, y - brush_size, x + brush_size, y + brush_size);
+               draw_oval(ibuf->drawing_area->window, ibuf->gc, 
+                       TRUE, x - brush_size, y - brush_size, x + brush_size, y + brush_size);
+               break;
+            default:
+               break;   
+         }
+      }
       changed = 1;
    }
    if (changed) { 
       IMAGE_MODIFIED;
    }
 
+   brush_distance = final;
+   brush_last_x = initial_x;
+   brush_last_y = initial_y; 
+   
    /* TODO: object-oriented drawing tools */
    if (ibuf->current_tool==ERASE)
    {
@@ -1337,8 +1382,6 @@ void image_buf_paint_interpolate(image_buf *ibuf, int x, int y)
    }
 }
 
-
-
 void image_buf_select_all(image_buf *ibuf)
 {
    long w, h;
